@@ -228,7 +228,7 @@ int process_request(struct ProfileData *pr, xmlDocPtr request, xmlDocPtr* respon
         msa_xmlDocDump(stdout, inner_req);
         
         /*unknown request */
-        if (xpath_get_node("//InnerRequest", inner_req) == NULL){
+        if (xpath_get_node("/InnerRequest", inner_req) == NULL){
             *response = xmlCopyDoc(inner_req, 1);
             xmlFreeDoc(inner_req);
             return 0;
@@ -239,7 +239,7 @@ int process_request(struct ProfileData *pr, xmlDocPtr request, xmlDocPtr* respon
             return 0;
         }
 
-        node = xpath_get_node("//InnerRequest", inner_req);
+        node = xpath_get_node("/InnerRequest", inner_req);
         if (node == NULL){
             xmlFreeDoc(inner_req);
             return 1;
@@ -247,11 +247,11 @@ int process_request(struct ProfileData *pr, xmlDocPtr request, xmlDocPtr* respon
         http_type = (gchar*)xmlGetProp(node, BAD_CAST "type");
 
         if (g_strcmp0(http_type, "post") == 0 || g_strcmp0(http_type, "get") ==0) {
-            node = xpath_get_node("//InnerRequest/url", inner_req);
+            node = xpath_get_node("/InnerRequest/url", inner_req);
             if (node != NULL)
                 tmp = (gchar*)xmlNodeGetContent(node);
 
-            /*sig_node = xpath_get_node("//InnerRequest/sig", inner_req);
+            /*sig_node = xpath_get_node("/InnerRequest/sig", inner_req);
             if (sig_node != NULL){
                 sig = (gchar*)xmlNodeGetContent(sig_node);
                 md5_sig = g_compute_checksum_for_string(G_CHECKSUM_MD5, sig, strlen(sig));
@@ -272,11 +272,11 @@ int process_request(struct ProfileData *pr, xmlDocPtr request, xmlDocPtr* respon
             if (g_strcmp0(http_type, "post") == 0) {
                 msa_debug("UPLOADING...");
                 file = (struct UpFile *)malloc(sizeof (struct UpFile));
-                node = xpath_get_node("//InnerRequest/post_param[@name='name']", inner_req);
+                node = xpath_get_node("/InnerRequest/post_param[@name='name']", inner_req);
                 file->name = (gchar*)xmlNodeGetContent(node);
-                node = xpath_get_node("//InnerRequest/post_param[@name='file']", inner_req);
+                node = xpath_get_node("/InnerRequest/post_param[@name='file']", inner_req);
                 file->file_path = (gchar*)xmlNodeGetContent(node);
-                node = xpath_get_node("//InnerRequest/post_param[@name='file_name']", inner_req);
+                node = xpath_get_node("/InnerRequest/post_param[@name='file_name']", inner_req);
                 file->file_name = (gchar*)xmlNodeGetContent(node);
                 
                 http_code = http_upload_file(url, info_proxy, info_port, NULL, &data, file);
@@ -299,7 +299,7 @@ int process_request(struct ProfileData *pr, xmlDocPtr request, xmlDocPtr* respon
         /*msa_xmlDocDump(stdout, inner_req);*/
         
         /*Parse server response*/
-        node = xpath_get_node("//InnerRequest/parser", inner_req);
+        node = xpath_get_node("/InnerRequest/parser", inner_req);
         if (node != NULL){
             parser_type = (gchar*)xmlGetProp(node, BAD_CAST "type");
             tmp = (gchar*)xmlNodeGetContent(node);
@@ -308,17 +308,20 @@ int process_request(struct ProfileData *pr, xmlDocPtr request, xmlDocPtr* respon
                 if (g_strcmp0(parser_type, "json") == 0) {
                     parse_upload(data, &resp_str);
                     g_free(data);
-                    data = g_strdup(resp_str);
-                    g_free(resp_str);
-                }
+                    data = resp_str;
+                    resp_str=NULL;
+                } else
                 if (g_strcmp0(parser_type, "settings") == 0) {
                     msa_debug("SETTINGS");
                     g_free(data);
-                    if (set_settings(inner_req, pr))
+                    if (set_settings(inner_req, pr)) {
+                        g_free(data);
                         data = g_strdup(MSG_SETTINGS_SAVED_ERROR);
-                    else
+                    } else {
+                        g_free(data);
                         data = g_strdup(MSG_SETTINGS_SAVED);
-                }
+                    }
+                } else
                 if (g_strcmp0(parser_type, "download") == 0) {
                     if (save_file (inner_req, data, data_size)) {
                         g_free(data);
@@ -430,6 +433,7 @@ int fb_req(xmlDocPtr reqXml, xmlDocPtr* doc, const struct msa_module* info)
     char* error_code = NULL;
     xmlChar* class_val = NULL;
     xmlNodePtr node = NULL;
+    gchar* no_auth = NULL;
 
     struct ProfileData *cp = (struct ProfileData *)profile_get_by_id(info->id);
     captch = 0;
@@ -443,10 +447,14 @@ int fb_req(xmlDocPtr reqXml, xmlDocPtr* doc, const struct msa_module* info)
     g_print("IN FB_REQ!!! %d\n", global_temp);
     counter = 0;
     msa_xmlDocDump(stdout, reqXml);
+    node = xpath_get_node("//Request", reqXml);
+    if(node != NULL)
+        no_auth = (gchar*) xmlGetProp(node, (xmlChar *)"noAuthorize");
 
     class_val = get_req_class(reqXml);
     if (g_strcmp0((char*) class_val, "settings") != 0)
-        init_status = initialization(cp);
+        if(no_auth == NULL || g_strcmp0(no_auth, "true") != 0)
+            init_status = initialization(cp);
     g_free(class_val);
 
     attempts = 0;
@@ -468,19 +476,32 @@ int fb_req(xmlDocPtr reqXml, xmlDocPtr* doc, const struct msa_module* info)
         return 0;
     }
 
+    node = xpath_get_node("//Response[@function != 'errorMessage']", *doc);
+    if (node != NULL){
+        if(init_status == 3) {
+            xmlSetProp(node, (xmlChar *)"authorized", (xmlChar *)"true");
+        }
+        if(init_status == 0) {
+            xmlSetProp(node, (xmlChar *)"authorized", (xmlChar *)"false");
+        }
+    }
+
     node = xpath_get_node("//Response[@function='errorMessage']/Params/string[@name='code']", *doc);
     if (node != NULL){
         msa_debug("error occured");
         error_code = (gchar *)xmlNodeGetContent(node);
-        if(g_strcmp0(error_code, DRV_AUTH_ERROR_CODE) == 0 
-            || g_strcmp0(error_code, "190") == 0){
-            if (authorize(cp) == 0) {
-                xmlFreeDoc(*doc);
-                process_request(cp, reqXml, doc);
+        if(g_strcmp0(error_code, DRV_AUTH_ERROR_CODE) == 0 || g_strcmp0(error_code, "190") == 0){
+            if(no_auth == NULL || g_strcmp0(no_auth, "true") != 0) {
+                if (authorize(cp) == 0) {
+                    xmlFreeDoc(*doc);
+                    process_request(cp, reqXml, doc);
+                    node = xpath_get_node("//Response[@function != 'errorMessage']", *doc);
+                    if (node != NULL)
+                        xmlSetProp(node, (xmlChar *)"authorized", (xmlChar *)"true");
+                }
             }
         }
         g_free(error_code);
-            
     }
     return 0;
 }
@@ -504,6 +525,7 @@ int initialization(struct ProfileData *cp)
             msa_debug("dr_fb: Could not get remixmid, remixsid, auth_status = %d \n", auth_status); 
             return auth_status;
         }
+        return 3;
     }
     msa_debug("init END");
     return 0;
